;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Fire Flower Disassembly
;
;This is a disassembly of sprite 75 in SMW - Fire Flower.
;
;By RussianMan. Credit is optional.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Extra Byte 1 - YES
;1  - It'll initially blink-fall
;2  - it'll initially start rising (as if it was rising out of block)
;3+ - It won't use any gravity (as if it was spawn out of flying ?-block)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;Defines and options

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Sound Effects:
;check https://www.smwcentral.net/?p=viewthread&t=6665 for more info on sound effects and banks
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!ItemBoxSound = $0B		;sound that is supposed to play when you pick it in your item box
!ItemBoxBank = $1DFC		;

!PickPowerupSound = $0A		;sound effect that always plays
!PickPowerupBank = $1DF9	;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Score:
;00 = 100
;01 = 200
;02 = 400
;03 = 800
;04 = 1000
;05 = 2000
;06 = 4000
;07 = 8000
;08 = 1up
;09 = 2up
;0A = 3up
;0B = 5up (may glitch)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!ScoreFlower = $04

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Speed Defines:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!BlinkFallSpeed = $10		;Y-Speed for dropped power-up

!RisingSpeed = $FC		;Y-Speed for "rising out of ?-block"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;GFX-Related:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!FlowerTile = $46

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Other:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ItemBox:            		;item box contents for each player powerup
db $00,$01,$04,$02


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Initial checks and settings
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Print "INIT ",pc
	RTL
Init:
LDA !extra_byte_1,x		;"pointers"
BEQ .NoINIT			;not true, it should enable "can't move" flag, but it's useless for the most part
DEC				;
BEQ .BlinkFall			;otherwise if it's 1, make it blink-fall
DEC				;
BEQ .Rising			;on 2 it'll be raising.
;LDA #$80			;
;STA !C2,x			;on 3 and more break gravity laws
INC !C2,x			;

.NoINIT
RTL				;

.Rising
LDA #$3E			;raising timer.
STA !1540,x			;
RTL

.BlinkFall
INC !1534,x			;
RTL				;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Main Code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Print "MAIN ",pc
PHB
PHK
PLB
JSR Flower
PLB
RTL

;most of the code comes from my mushrooms disassembly, because some power-ups in original SMW share same code.
speed_table: db $E0,$20

Flower:				;
LDA $14				;turn left-right
AND #$08			;
LSR #3				;
STA !157C,x			;

LDA $64				;Yes, I copy-pasted my another power-up disassembly's code
PHA				;because power-ups share same code
JSR .Interact			;At the same time I feel it's better for each of them to be stand-alone sprite.

LDA !sprite_status,x
CMP #$09
BCC +
LDA #$08
STA !sprite_status,x

LDA $76
AND #$01
TAY
LDA speed_table,y
STA !B6,x
+



LDA !1534,x			;check if it's blink-falling
BEQ .NoBlinkFall		;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Code for Blink-Fall state
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

LDA $9D				;yeah, second 9D check in this whole code.
ORA $7F8331
BNE .NoMove			;I know. It's no_good.
LDA #!BlinkFallSpeed		;
STA !AA,x			;Y-speed
JSL $01801A|!BankB		;

.NoMove
LDA $14				;show GFX
AND #$0C			;every 13 frames
BNE .GFX			;
PLA				;
RTS				;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.NoBlinkFall
LDA $9D				;don't do things on 9D
ORA $7F8331
BNE .GFX			;moved this here so we don't have multiple 9D checks

LDA !1540,x			;check if it's rising out of block
BEQ .NotRising			;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Code for Rising Out Of Block state
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;JSL $019138|!BankB		;it's rising. interact with objects, even though IDK why it's needed. It doesn't use any interaction code.

LDA !1528,x			;
BNE .NotBehind			;

LDA #$10			;low priority
STA $64				;

.NotBehind
LDA #!RisingSpeed		;
STA !AA,x			;
JSL $01801A|!BankB		;

.GFX
JMP .NoSide			;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Code for when Flower is in normal state
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.NotRising
INC !1570,x			;must have for power-up. so when yoshi swallows sprite, it'll give bonus, depending on acts as setting.
;JSR XSpeed			;don't handle X-speed

;LDA !9E,x			;if it isn't fire flower
;CMP #$75			;
;BNE .NoResetX			;This time it's true

;LDA !151C,x			;
;BNE .NoResetX			;

LDA !B6,x			;no X-speed
BEQ .skip_x
BMI +
DEC
BRA ++
+
INC
++
STA !B6,x
.skip_x

.NoResetX

;CMP #$76			;
;BEQ .NoASL			;
;CMP #$21			;
;BEQ .NoASL			;
;LDA !151C,x			;
;BNE .NoASL			;

;ASL !B6,x			;Not needed, X-speed is always zero

.NoASL
LDA !C2,x			;This doesn't works and Fire flower doesn't moves anyways (it can only move vertically)
;BEQ .Move			;move
;BMI .Label2			;however when spawned out of flying ?-block sprite it's high bit is set, which disables gravity
BNE .Label2			;I've made it like this for small optimization.

;JSL $019138|!BankB		;
;LDA !1588,x			;if it isn't in contact with any object in any way
;BNE .Label2			;
;STZ !C2,x			;reset movement flag, so it can move

;BRA .Label2

.Move

;Mode 7 checks

.UpdateThings
JSL $01802A|!BankB		;

LDA $13				;
AND #$03			;every 3rd frame
BEQ .Label2			;
DEC !AA,x			;decrease Y speed

.Label2
LDA #$00			;handle Offscreen
%SubOffScreen()			;

LDA !1588,x			;check for ceiling
AND #$08			;
BEQ .NoYReset			;reset Y-speed when touching ceiling

;LDA #$00			;:angry:
;STA !AA,x			;

STZ !AA,x			;:smile:

.NoYReset

LDA !1588,x			;
AND #$04			;
BNE .AAA			;
BRA .Continue_checking		;

.AAA
LDA !1588,x			;
BMI .Speed1			;
LDA #$00			;
LDY !15B8,x			;
BEQ .Speed2			;

.Speed1
LDA #$18			;

.Speed2
STA !AA,x			;

.Continue_checking
;LDA !1558,x			;this is unused, supposed to disable interaction with walls
;LDA !C2,x			;
;BNE .NoSide			;double unneeded

.NoSide

LDA !1540,x			;if mushroom's rising out of block
CMP #$36			;
BCS .RestorePriority		;for a few frames don't mess with priority

;LDA !C2,x			;if it moves, don't hide it
;BEQ .NotStationary		;forgot to comment out
;CMP #$FF			;check if all bits set
;BNE .SlightPriority		;it'll be checking for "behind scenery" flag (in our case it can't be FF)
;BRA .SlightPriority		;

.NotStationary
LDA !1632,x			;
BEQ .JSR			;

.SlightPriority
LDA #$10			;
STA $64				;

.JSR
JSR .FinallyGFX			;finally, do GFX things!

.RestorePriority
PLA				;self-explanatory.
STA $64				;
RTS				;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Handle Interaction
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.Interact
JSR .Interaction		;
BCC .Ret			;

LDA !154C,x			;if it can't interact
BNE .Ret			;don't

LDA !1540,x			;if rising out of block
CMP #$18			;don't interact
BCS .Ret			;this creates glitch where you can eat power-up and not recieve it.

LDA $1497|!Base2
BNE .Ret

STZ !14C8,x			;sprite begone

	LDA $1490|!Base2
	BEQ .no_star

	%Star()

.no_star

	JSL $00F5B7|!BankB
	
.Ret
RTS				;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;GFX Routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.FinallyGFX
%GetDrawInfo()			;

;LDA $140F|!Base2		;I don't think this is usefull for regular user
;BNE .NotMode7			;I don't think you normally would put this sprite in Mode 7 rooms.
;LDA $0D9B|!Base2		;but if you know this IS going to be usefull (for some reason), uncomment these
;CMP #$C1			;those checks if sprite's in any mode 7 room.
;BEQ .NotMove7			;
;BIT $0D9B|!Base2		;
;BVC .NotMode7

;LDA #$D8			;set specific OAM slot for power-up
;STA !15EA,x			;
;TAY

.NotMode7
LDA $00				;
STA $0300|!Base2,y		;Sprite tile's X position

LDA $01				;
DEC				;Sprite tile's Y position - 1, so it looks better on ground
STA $0301|!Base2,y		;

LDA !157C,x			;
LSR				;
LDA #$00			;
BCS .NoFlip			;
ORA #$40			;

.NoFlip
ORA $64				;
ORA !15F6,x			;
STA $0303|!Base2,y		;

LDA #!FlowerTile		;Fire Flower Tile
STA $0302|!Base2,y		;

LDY #$02			;
LDA #$00			;
JSL $01B7B3|!BankB		;
RTS				;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Subroutine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.Interaction
PHK				;btw my second time using this jump method
PEA.w .Re-1			;I hope it doesn't sucks. again.
PEA.w $0180CA|!BankB-1		;I'm not an idiot, I'm just learning. again.
JML $01A80F|!BankB		;and maybe I am an idiot. again.

.Re
RTS				;